﻿using System.Windows;
using System.Windows.Controls;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Benchmark.Utils;

namespace Dapfor.Wpf.Grid.Benchmark
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            logger.RowAdded += (sender, e) => e.Row.EnsureVisible();

            logger.ItemsSource = Logger.Messages;
            logger.Headers[0]["Date"].SortDirection = SortDirection.Ascending;

            foreach (TabItem item in tabControl1.Items)
            {
                Control control = (Control) item.Content;
                control.ClearValue(WidthProperty);
                control.ClearValue(HeightProperty);
            }
        }

        private void OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            foreach (var addedItem in e.AddedItems)
            {
                TabItem item = addedItem as TabItem;
                IActivable activable = item != null ? item.Content as IActivable : null;
                if(activable != null)
                {
                    activable.Activate();
                }
            }

            foreach (var removedItem in e.RemovedItems)
            {
                TabItem item = removedItem as TabItem;
                IActivable activable = item != null ? item.Content as IActivable : null;
                if (activable != null)
                {
                    activable.Deactivate();
                }
            }
        }
    }
}
