﻿using System;
using System.Windows;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Benchmark.Utils;

namespace Dapfor.Wpf.Grid.Benchmark
{
    /// <summary>
    /// Interaction logic for RemoveRowControl.xaml
    /// </summary>
    public partial class RemoveRowControl : IActivable
    {
        private static readonly HighPerformanceTimer HiPerfTimer = new HighPerformanceTimer();
        private bool _stopping;

        public RemoveRowControl()
        {
            InitializeComponent();
        }

        public void Activate()
        {
            panel.IsEnabled = true;
        }

        public void Deactivate()
        {
            _stopping = true;
            panel.IsEnabled = false;
        }

        private void OnStartButtonClick(object sender, RoutedEventArgs e)
        {

            Random random = new Random();
            panel.IsEnabled = false;

            //Remove sort
            grid.Headers[0].SortedColumns.Clear();

            int rowCount = 500000;
            int tranche = 10000;

            //Limit amount of data
            if (rbFromEnd.IsChecked == true)
            {
                Logger.Log(string.Format("Performance measurement when rows are removed from the end of grid"));
                rowCount = 500000;
                tranche = 10000;
            }
            else if (rbFromBeginning.IsChecked == true)
            {
                Logger.Log(string.Format("Performance measurement when rows are removed from the beginning of grid"));
                rowCount = 100000;
                tranche = 10000;
            }
            else if (rbFromSortedGrid.IsChecked == true)
            {
                Logger.Log(string.Format("Performance measurement when rows are removed from sorted grid"));
                rowCount = 100000;
                tranche = 10000;
            }

            //Populate the grid
            _stopping = false;
            if (grid.Rows.Count == 0)
            {
                Logger.Log(string.Format("Prepare Wpf Grid and populate it with {0} rows...", rowCount));
                Helper.DoEvents();
                for (int i = 0; (i < rowCount) && !_stopping; ++i)
                {
                    grid.Rows.Add(new TestDataObject());
                }
            }

            //Enable sort if needed
            if (rbFromSortedGrid.IsChecked == true)
            {
                Logger.Log(string.Format("Sort {0} rows...", rowCount));
                Helper.DoEvents();
                grid.Headers[0][1].SortDirection = SortDirection.Ascending;
                grid.Headers[0][3].SortDirection = SortDirection.Descending;

                Logger.Log(string.Format("Now remove them from random positions..."));
                Helper.DoEvents();
            }

            HiPerfTimer.Start();

            for (int i = grid.Rows.Count - 1; (i >= 0) && !_stopping; --i)
            {
                if (rbFromEnd.IsChecked == true)
                {
                    //Remove from the end
                    grid.Rows[grid.Rows.Count - 1].Remove();
                }
                else if (rbFromBeginning.IsChecked == true)
                {
                    //From the beginning
                    grid.Rows[0].Remove();
                }
                else
                {
                    //Remove random row from the sorted grid
                    int index = random.Next()%grid.Rows.Count;
                    grid.Rows[index].Remove();
                }

                if (i % tranche == 0)
                {
                    HiPerfTimer.Stop();


                    double duration = HiPerfTimer.Duration;

                    int rate = duration > 0 ? (int)(1000 * tranche / duration) : 0;

                    if (rbFromEnd.IsChecked == true)
                    {
                        Logger.Log(string.Format("{2} rows ({0} - {1}) removed from the end in {3:0} msec. Rate: {4} rows/sec", i, i + tranche - 1, tranche, duration, rate));
                    }
                    else if (rbFromBeginning.IsChecked == true)
                    {
                        Logger.Log(string.Format("Grid of {0} rows: first {1} rows removed from the beginning in {2:0} msec. Rate: {3} rows/sec", grid.Rows.Count + tranche, tranche, duration, rate));
                    }
                    else
                    {
                        //Sorted grid
                        Logger.Log(string.Format("Grid of {0} rows: random {1} rows removed from the sorted grid in {2:0} msec. Rate: {3} rows/sec", grid.Rows.Count + tranche, tranche, duration, rate));
                    }

                    Helper.DoEvents();

                    HiPerfTimer.Start();
                }
            }

            HiPerfTimer.Stop();

            panel.IsEnabled = true;
            Logger.Log(string.Format("Removing finished"));
        }

        
    }
}
