﻿using System;
using System.Windows;
using System.Windows.Threading;
using Dapfor.Wpf.Controls;

namespace Dapfor.Wpf.Grid.Benchmark.Utils
{
    internal class LoadBalancingGrid : GridControl
    {
        private bool _isLimitFound;
        private TimeSpan _lastPeriodBetweenTicks;
        private DateTime _lastTick;
        private long _limitCount;
        private int _multiplier = 1;
        private long _tickCount;
        private readonly DispatcherTimer _timer = new DispatcherTimer();

        static LoadBalancingGrid()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(LoadBalancingGrid), new FrameworkPropertyMetadata(typeof(LoadBalancingGrid)));
            //StyleProperty.OverrideMetadata(typeof(LoadBalancingGrid), new FrameworkPropertyMetadata(typeof(GridControl)));
        }

        public LoadBalancingGrid()
        {
            _timer.Interval = TimeSpan.FromMilliseconds(10);
            _lastTick = DateTime.UtcNow;
            Dispatcher.ShutdownStarted += Dispatcher_ShutdownStarted;
            _timer.Tick += OnTimerTick;

            
        }

        private void Dispatcher_ShutdownStarted(object sender, EventArgs e)
        {
            
            
        }

        public void StartLoadBalancingProcess()
        {
            _tickCount = 0;
            _limitCount = 0;
            _multiplier = 1;
            _isLimitFound = false;
            _timer.Start();
        }

        public void StopLoadBalancingProcess()
        {
            _tickCount = 0;
            _limitCount = 0;
            _multiplier = 1;
            _isLimitFound = false;
            _timer.Stop();
        }

        public event EventHandler ProcessUpdate;

        private void OnTimerTick(object sender, EventArgs e)
        {

            for (int i = 0; i < _multiplier; ++i)
            {
                if (ProcessUpdate != null)
                {
                    //If this method is called too often, than the distance between two timer ticks will incrase
                    ProcessUpdate(this, EventArgs.Empty);
                }
            }

            //Compute a period between two timer ticks - if it is too big - the Application works at the maximum of CPU resources
            _lastPeriodBetweenTicks = DateTime.UtcNow - _lastTick;
            _lastTick = DateTime.UtcNow;

            //Do 10 empty cycles
            if (_tickCount++ > 10)
            {
                //Balance the speed
                AdjustRefreshSpeed();
            }
        }



        public bool IsLimitFound
        {
            get { return _isLimitFound; }
        }

        private void AdjustRefreshSpeed()
        {
            if (_lastPeriodBetweenTicks.TotalMilliseconds > 5 * _timer.Interval.TotalMilliseconds)
            {
                if (_limitCount++ > 5)
                {
                    _isLimitFound = true;
                }
            }

            //Start load balancing
            if (_isLimitFound)
            {
                //The period between two timer ticks is too big - decrase the speed
                if (_lastPeriodBetweenTicks.TotalMilliseconds > 2 * _timer.Interval.TotalMilliseconds)
                {
                    _multiplier -= 1;
                }
                else
                {
                    //The period between two timer ticks is resonable - incrase the speed
                    _multiplier += 1;
                }
                _multiplier = Math.Max(_multiplier, 0);
            }
            else
            {
                //Look for a limit
                _multiplier += (3 + (_multiplier/20));
            }
        }
    }
}